@@ -28,7 +28,9 @@ class DPTValue1ByteUnsigned(DPTBase):
         value = raw[0]
 
         if not cls._test_boundaries(value):
-            raise ConversionError("Cant parse %s" % cls.__name__, value=value, raw=raw)
+            raise ConversionError(
+                "Could not parse %s" % cls.__name__, value=value, raw=raw
+            )
 
         return value
 
@@ -41,7 +43,7 @@ class DPTValue1ByteUnsigned(DPTBase):
                 raise ValueError
             return (knx_value,)
         except ValueError:
-            raise ConversionError("Cant serialize %s" % cls.__name__, value=value)
+            raise ConversionError("Could not serialize %s" % cls.__name__, value=value)
 
     @classmethod
     def _test_boundaries(cls, value):
@@ -122,7 +124,9 @@ class DPTSceneNumber(DPTValue1ByteUnsigned):
         value = raw[0] + 1
 
         if not cls._test_boundaries(value):
-            raise ConversionError("Cant parse %s" % cls.__name__, value=value, raw=raw)
+            raise ConversionError(
+                "Could not parse %s" % cls.__name__, value=value, raw=raw
+            )
 
         return value
 
@@ -135,4 +139,4 @@ class DPTSceneNumber(DPTValue1ByteUnsigned):
                 raise ValueError
             return (knx_value,)
         except ValueError:
-            raise ConversionError("Cant serialize %s" % cls.__name__, value=value)
+            raise ConversionError("Could not serialize %s" % cls.__name__, value=value)
